/**
 * This interface describes an adapter to load / save databases. It is required for the {@link SyncDatabase} class
 *
 * @author Nicolas Schmidt
 *
 * @see SyncDatabase - The user of this interface
 * @see SyncDatabaseAdapter.save - 💾 the save method of the SyncDatabaseAdapter
 * @see SyncDatabaseAdapter.load - 💾 the save method of the SyncDatabaseAdapter
 */
export interface SyncDatabaseAdapter {
    /**
     * 💾 Saves the given data
     *
     * @author Nicolas Schmidt
     * @param data 💾 the data to save
     *
     * @see DatabaseAdapter - 👩‍👦 the parent interface
     * @see DatabaseAdapter.load - 💾 the load function
     *
     */
    save(data: JSObject): void;
    /**
     * 💾 loads data
     *
     * @author Nicolas Schmidt
     * @return 💾 loads the data
     *
     * @see DatabaseAdapter - 👩‍👦 the parent interface
     * @see DatabaseAdapter.save - 💾 the save function
     */
    load(): JSObject;
}
/**
 * This interface describes an adapter to load / save databases. It is required for the {@link Database} class
 *
 * @author Nicolas Schmidt
 *
 * @see Database - The user of this interface
 * @see DatabaseAdapter.save - 💾 the save method of the DatabaseAdapter
 * @see DatabaseAdapter.load - 💾 the save method of the DatabaseAdapter
 */
export interface DatabaseAdapter {
    /**
     * 💾 Saves the given data
     *
     * @author Nicolas Schmidt
     * @param data 💾 the data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @see DatabaseAdapter - 👩‍👦 the parent interface
     * @see DatabaseAdapter.load - 💾 the load function
     *
     */
    save(data: JSObject): Promise<void>;
    /**
     * 💾 loads the data
     *
     * @author Nicolas Schmidt
     * @return ⌛ Promise (Ready when data is loaded) >> 💾 the data
     *
     * @see DatabaseAdapter - 👩‍👦 the parent interface
     * @see DatabaseAdapter.save - 💾 the save function
     */
    load(): Promise<JSObject>;
}
/**
 * 🔨 Creates an DatabaseArray from a given array
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the value to create a database-value from
 * @returns 🔨 the DatabaseArray representation of the given array
 *
 * @see DatabaseArray
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: any[]): DatabaseArray;
/**
 * 🔨 Creates an DatabaseObject from a given object
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the object to create a "DatabaseObject" from
 * @returns 🔨 the DatabaseObject-representation of the given object
 *
 * @see DatabaseObject
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: JSObject): DatabaseObject;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a DatabaseObject as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the value to create a database-value from
 * @returns ⛔ the same DatabaseObject that is given as input
 *
 * @see DatabaseObject
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: DatabaseObject): DatabaseObject;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a DatabaseArray as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the value to create a database-value from
 * @returns ⛔ the same DatabaseArray that is given as input
 *
 * @see DatabaseArray
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: DatabaseArray): DatabaseArray;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a string as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the value to create a database-value from
 * @returns ⛔ the same string that is given as input
 *
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: string): string;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a number as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 1️⃣ the value to create a database-value from
 * @returns ⛔ the same number that is given as input
 *
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: number): number;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a boolean as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ the value to create a database-value from
 * @returns ⛔ the same boolean that is given as input
 *
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: boolean): boolean;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * null as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ null
 * @returns ⛔ null
 *
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: null): null;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * undefined as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ undefined
 * @returns ⛔ undefined
 *
 * @see createDatabaseValue
 *
 */
export declare function createDatabaseValue(val: undefined): undefined;
/**
 * Creates a DatabaseValue from a given value
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the value to create a database-value from
 * @returns the database-value
 *
 */
export declare function createDatabaseValue(val: DatabaseInsertable): DatabaseValue;
/**
 * 🔨 Creates a normal array from a given DatabaseArray
 *
 * @author Nicolas Schmidt
 * @param val 📋 the DatabaseArray you want to convert back to a normal value
 * @returns 🔨 the simple array
 *
 * @see DatabaseObject
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: DatabaseArray): any[];
/**
 * 🔨 Creates a normal object from a given DatabaseObject
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the DatabaseObject you want to convert back to a normal value
 * @returns 🔨 the simple object
 *
 * @see DatabaseObject
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: DatabaseObject): JSObject;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a array as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the array you want to convert back to a normal value
 * @returns ⛔ the same array that was given as input
 *
 * @see DatabaseObject
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: any[]): any[];
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a object as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the object you want to convert back to a normal value
 * @returns ⛔ the same object that was given as input
 *
 * @see DatabaseObject
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: JSObject): JSObject;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a string as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the string you want to convert back to a normal value
 * @returns ⛔ the same string that was given as input
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: string): string;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a number as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 1️⃣ the number you want to convert back to a normal value
 * @returns ⛔ the same number that was given as input
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: number): number;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a boolean as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ the boolean you want to convert back to a normal value
 * @returns ⛔ the same boolean that was given as input
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: boolean): boolean;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * null as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ null
 * @returns ⛔ null
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: null): null;
/**
 * This function is not realy usefull. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * undefined as argument.
 *
 * @author Nicolas Schmidt
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ undefined
 * @returns ⛔ undefined
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: undefined): undefined;
/**
 * creates the 'normal' value from database-objects
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the value you want to convert back to a normal value
 * @returns the 'normal' value
 *
 * @see getNormalValue
 *
 */
export declare function getNormalValue(val: DatabaseInsertable): DatabaseValueAble;
/**
 * A DatabaseArray is the database-representation of an array.
 *
 * @author Nicolas Schmidt
 *
 * @see DatabaseArray.data
 * @see DatabaseArray.constructor
 * @see DatabaseArray.set - sets a value from the DatabaseArray
 * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
 * @see DatabaseArray.get - gets a value from the DatabaseArray
 * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
 * @see DatabaseArray.update - updates a value from the DatabaseArray
 * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
 * @see DatabaseArray.push - push a value into the DatabaseArray
 */
export declare class DatabaseArray {
    /**
     * The real array-contents of the DatabaseArray
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the getter and setter for the data
     *
     */
    protected _data: any[];
    /**
     * The data of the DatabaseArray
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray._data - the storage for the data
     *
     */
    set data(data: DatabaseInsertable[]);
    get data(): DatabaseInsertable[];
    /**
     * Returns the length of the array
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the data of the array
     *
     */
    get length(): number;
    /**
     * Constructor for DatabaseArray
     *
     * @author Nicolas Schmidt
     * @param data the data for the database-array
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the argument you give to the constructor
     *
     */
    constructor(data?: Array<any>);
    /**
     * Sets a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    set(key: number | string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets a default value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    setDefault(key: number | string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Gets a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value of the key
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    get(key: number | string | Array<string | number>): DatabaseValue;
    /**
     * Gets a value from the DatabaseArray normalizes it
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value of the key
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    getNormal(key: number | string | Array<string | number>): DatabaseValueAble;
    /**
     * Upates a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to update
     * @param update the funciton to update the value
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    update(key: number | string | Array<string | number>, update: (e: DatabaseValue) => DatabaseInsertable): this;
    /**
     * Checks if the DatabaseArray contains a value
     *
     * @author Nicolas Schmidt
     * @param key the path to check
     * @returns does the array contains the value
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseObject
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseObject
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    contains(key: number | string | Array<string | number>): boolean;
    /**
     * Pushs a value into the DatabaseArray
     *
     * @author Nicolas Schmidt
     * @param value the value to push into the array
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     *
     */
    push(...values: DatabaseInsertable[]): this;
}
/**
 * A DatabaseObject is the database-representation of an object.
 *
 * @author Nicolas Schmidt
 *
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
export declare class DatabaseObject {
    /**
     * The real object-contents of the DatabaseObject
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.data - the getter and setter for the data
     *
     */
    protected _data: JSObject;
    /**
     * The data of the DatabaseObject
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject._data - the storage for the data
     *
     */
    set data(data: JSObject);
    get data(): JSObject;
    /**
     * Returns the length of the object
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the data of the array
     *
     */
    get length(): number;
    /**
     * Constructor for DatabaseObject
     *
     * @author Nicolas Schmidt
     * @param data the data for the database-array
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.data - the argument you give to the constructor
     *
     */
    constructor(data?: JSObject);
    /**
     * Sets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.getNormal - gets a normalized value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    set(key: string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets a default value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    setDefault(key: string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets default values for the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param defaults an object containing all the default values
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    setDefaults(defaults: JSObject): this;
    /**
     * Gets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value at the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.getNormal - gets a normalized value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    get(key: string | Array<string | number>): DatabaseValue;
    /**
     * Gets a value from the DatabaseObject and normalizes it
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the normalized value at the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    getNormal(key: string | Array<string | number>): DatabaseValueAble;
    /**
     * Updates a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to update
     * @param update the function to update the value
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    update(key: string | Array<string | number>, update: (e: DatabaseValue) => DatabaseInsertable): this;
    generateId(name: string): number;
    /**
     * Checks if the DatabaseObject contains a value
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @returns the value at the position of the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     *
     */
    contains(key: string | Array<string | number>): boolean;
}
/**
 * Database class is basically a DatabaseObject with save and load functionality
 *
 * @see SyncDatabase
 * @see Database.saveData
 * @see Database.reloadData
 *
 * @see DatabaseObject
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
export declare class Database extends DatabaseObject {
    adapter: DatabaseAdapter;
    /**
     * Constructor for Database class
     *
     * @author Nicolas Schmidt
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see Database - 👩‍👦 the parent class
     */
    constructor(adapter: DatabaseAdapter, data: Object);
    /**
     * Save the data from the database
     *
     * @author Nicolas Schmidt
     * @returns ⌛ Promise (Ready when saved) >> the database itself, so you can chain operations
     */
    saveData(): Promise<this>;
    /**
     * Reload the data from the database
     *
     * @author Nicolas Schmidt
     * @returns ⌛ Promise (Ready when loaded) >> the database itself, so you can chain operations
     */
    reloadData(): Promise<this>;
}
/**
 * SyncDatabase is a database that does not use promises.
 *
 * @see Database
 * @see SyncDatabase.saveData
 * @see SyncDatabase.reloadData
 *
 * @see DatabaseObject
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
export declare class SyncDatabase extends DatabaseObject {
    adapter: SyncDatabaseAdapter;
    /**
     * Constructor for Database class
     *
     * @author Nicolas Schmidt
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see Database - 👩‍👦 the parent class
     */
    constructor(adapter: SyncDatabaseAdapter, data?: Object);
    /**
     * Save the data from the database
     *
     * @author Nicolas Schmidt
     * @returns the database itself, so you can chain operations
     */
    saveData(): this;
    /**
     * Reload the data from the database
     *
     * @author Nicolas Schmidt
     * @returns the database itself, so you can chain operations
     */
    reloadData(): this;
}
/**
 * Creates a database from a given adapter
 *
 * @author Nicolas Schmidt
 * @param adapter the databaseAdapter
 * @returns ⌛ Promise (Ready when loaded) >> the database
 *
 */
export declare function createDatabase(adapter: DatabaseAdapter): Promise<Database>;
/**
 * Creates a sync-database from a given adapter
 *
 * @author Nicolas Schmidt
 * @param adapter the databaseAdapter
 * @returns the SyncDatabase
 *
 */
export declare function createDatabase(adapter: SyncDatabaseAdapter): SyncDatabase;
/**
 * Creates a database from a given adapter
 *
 * @author Nicolas Schmidt
 * @param adapter the databaseAdapter
 * @returns the Database
 *
 */
export declare function createDatabase(adapter: DatabaseAdapter | SyncDatabaseAdapter): Promise<Database> | SyncDatabase;
export declare type DatabaseValue = DatabaseObject | DatabaseArray | number | string | boolean | null | undefined;
export declare type DatabaseValueAble = any[] | JSObject | number | string | boolean | null | undefined;
export declare type DatabaseInsertable = DatabaseValue | DatabaseValueAble;
export interface JSObject {
    [key: string]: any;
}
