"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDatabase = exports.SyncDatabase = exports.Database = exports.DatabaseObject = exports.DatabaseArray = exports.getNormalValue = exports.createDatabaseValue = void 0;
/**
 * Implementation for the createDatabaseValue function
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the value to create a database-value from
 * @returns the database-value
 *
 */
function createDatabaseValue(val) {
    if (val instanceof DatabaseObject || val instanceof DatabaseArray)
        return val;
    if (val instanceof Array)
        return new DatabaseArray(val);
    if (val instanceof Object)
        return new DatabaseObject(val);
    return val;
}
exports.createDatabaseValue = createDatabaseValue;
/**
 * [implementation]
 * creates the 'normal' value from database-objects
 *
 * @author Nicolas Schmidt
 * @param val 🏡 the value you want to convert back to a normal value
 * @returns the 'normal' value
 *
 * @see getNormalValue
 *
 */
function getNormalValue(val) {
    if (val instanceof DatabaseObject || val instanceof DatabaseArray)
        return val.data;
    else
        return val;
}
exports.getNormalValue = getNormalValue;
/**
 * A DatabaseArray is the database-representation of an array.
 *
 * @author Nicolas Schmidt
 *
 * @see DatabaseArray.data
 * @see DatabaseArray.constructor
 * @see DatabaseArray.set - sets a value from the DatabaseArray
 * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
 * @see DatabaseArray.get - gets a value from the DatabaseArray
 * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
 * @see DatabaseArray.update - updates a value from the DatabaseArray
 * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
 * @see DatabaseArray.push - push a value into the DatabaseArray
 */
class DatabaseArray {
    /**
     * Constructor for DatabaseArray
     *
     * @author Nicolas Schmidt
     * @param data the data for the database-array
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the argument you give to the constructor
     *
     */
    constructor(data = []) { this.data = data; }
    /**
     * The data of the DatabaseArray
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray._data - the storage for the data
     *
     */
    set data(data) {
        this._data = [];
        data.forEach((v, k) => this._data[k] = createDatabaseValue(v));
    }
    get data() {
        let data = [];
        for (let i = 0; i < this._data.length; i++)
            data[i] = getNormalValue(this._data[i]);
        return data;
    }
    /**
     * Returns the length of the array
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the data of the array
     *
     */
    get length() {
        return this._data.length;
    }
    /**
     * Sets a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    set(key, value) {
        if (typeof key == "string")
            this.set(key.split(/[.\[\]]/g), value);
        else if (typeof key == "number")
            this._data[key] = createDatabaseValue(value);
        else {
            if (key.length > 1) {
                if (this._data[key[0]] == null)
                    this._data[key[0]] = createObjectForKey(key);
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    throw new Error(`Can't use set property of ${typeof e}`);
                e.set(key.slice(1, key.length), value);
            }
            else {
                this._data[typeof key[0] == "string" ? parseInt(key[0]) : key[0]] = createDatabaseValue(value);
            }
        }
        return this;
    }
    /**
     * Sets a default value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    setDefault(key, value) {
        if (!this.contains(key))
            this.set(key, value);
        return this;
    }
    /**
     * Gets a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value of the key
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    get(key) {
        if (typeof key == "string")
            return this.get(key.split(/[.\[\]]/g));
        else if (typeof key == "number")
            return this._data[key];
        else {
            if (key.length > 1) {
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    return undefined;
                return e === null || e === void 0 ? void 0 : e.get(key.slice(1, key.length));
            }
            else {
                return this._data[typeof key[0] == "string" ? parseInt(key[0]) : key[0]];
            }
        }
    }
    /**
     * Gets a value from the DatabaseArray normalizes it
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value of the key
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    getNormal(key) {
        return getNormalValue(this.get(key));
    }
    /**
     * Upates a value from the DatabaseArray.
     *
     * @author Nicolas Schmidt
     * @param key the path to update
     * @param update the funciton to update the value
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    update(key, update) {
        this.set(key, update(this.get(key)));
        return this;
    }
    /**
     * Checks if the DatabaseArray contains a value
     *
     * @author Nicolas Schmidt
     * @param key the path to check
     * @returns does the array contains the value
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseObject
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseObject
     * @see DatabaseArray.push - push a value into the DatabaseArray
     *
     */
    contains(key) {
        if (typeof key == "string")
            return this.contains(key.split(/[.\[\]]/g));
        else if (typeof key == "number")
            return this._data[key] != null;
        else {
            if (key.length > 1) {
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    return false;
                return e === null || e === void 0 ? void 0 : e.contains(key.slice(1, key.length));
            }
            else {
                return (this._data[typeof key[0] == "string" ? parseInt(key[0]) : key[0]]) != null;
            }
        }
    }
    /**
     * Pushs a value into the DatabaseArray
     *
     * @author Nicolas Schmidt
     * @param value the value to push into the array
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.set - sets a value from the DatabaseArray
     * @see DatabaseArray.setDefault - sets default a value from the DatabaseArray
     * @see DatabaseArray.get - gets a value from the DatabaseArray
     * @see DatabaseArray.getNormal - gets a value from the DatabaseArray and normalizes it
     * @see DatabaseArray.update - updates a value from the DatabaseArray
     * @see DatabaseArray.contains - checks if the DatabaseArray contains a value
     *
     */
    push(...values) {
        this._data.push(...values.map(v => createDatabaseValue(v)));
        return this;
    }
}
exports.DatabaseArray = DatabaseArray;
/**
 * A DatabaseObject is the database-representation of an object.
 *
 * @author Nicolas Schmidt
 *
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
class DatabaseObject {
    /**
     * Constructor for DatabaseObject
     *
     * @author Nicolas Schmidt
     * @param data the data for the database-array
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.data - the argument you give to the constructor
     *
     */
    constructor(data = {}) { this.data = data; }
    /**
     * The data of the DatabaseObject
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject._data - the storage for the data
     *
     */
    set data(data) {
        this._data = [];
        Object.keys(data).forEach(k => this._data[k] = createDatabaseValue(data[k]));
    }
    get data() {
        let data = {};
        Object.keys(this._data).forEach(k => data[k] = getNormalValue(this._data[k]));
        return data;
    }
    /**
     * Returns the length of the object
     *
     * @author Nicolas Schmidt
     *
     * @see DatabaseArray - 👩‍👦 the parent class
     * @see DatabaseArray.data - the data of the array
     *
     */
    get length() {
        return Object.keys(this._data).length;
    }
    /**
     * Sets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.getNormal - gets a normalized value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    set(key, value) {
        if (typeof key == "string")
            return this.set(key.split(/[.\[\]]/g), value);
        else {
            if (key.length > 1) {
                if (this._data[key[0]] == null)
                    this._data[key[0]] = createObjectForKey(key);
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    throw new Error(`Can't use set property of ${typeof e}`);
                e.set(key.slice(1, key.length), value);
            }
            else {
                this._data[key[0]] = createDatabaseValue(value);
            }
        }
        return this;
    }
    /**
     * Sets a default value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    setDefault(key, value) {
        if (!this.contains(key))
            this.set(key, value);
        return this;
    }
    /**
     * Sets default values for the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param defaults an object containing all the default values
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    setDefaults(defaults) {
        Object.keys(defaults).forEach(k => {
            if (!this._data[k])
                this._data[k] = createDatabaseValue(defaults[k]);
            else if (defaults[k] instanceof Object && this._data[k] instanceof DatabaseObject)
                this._data[k].setDefaults(defaults[k]);
        });
        return this;
    }
    /**
     * Gets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the value at the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.getNormal - gets a normalized value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    get(key) {
        if (typeof key == "string")
            return this.get(key.split(/[.\[\]]/g));
        else {
            if (key.length > 1) {
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    return undefined;
                return e === null || e === void 0 ? void 0 : e.get(key.slice(1, key.length));
            }
            else {
                return this._data[key[0]];
            }
        }
    }
    /**
     * Gets a value from the DatabaseObject and normalizes it
     *
     * @author Nicolas Schmidt
     * @param key the path to get
     * @returns the normalized value at the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    getNormal(key) {
        return getNormalValue(this.get(key));
    }
    /**
     * Updates a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt
     * @param key the path to update
     * @param update the function to update the value
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.contains - checks if the DatabaseObject contains a value
     *
     */
    update(key, update) {
        this.set(key, update(this.get(key)));
        return this;
    }
    generateId(name) {
        let counter = `id_counters.${name}`;
        if (this.get(counter) == null)
            this.set(counter, -1);
        return this.update(counter, (e) => e + 1).get(counter);
    }
    /**
     * Checks if the DatabaseObject contains a value
     *
     * @author Nicolas Schmidt
     * @param key the path to apply the value to
     * @returns the value at the position of the position
     *
     * @see DatabaseObject - 👩‍👦 the parent class
     * @see DatabaseObject.set - sets a value from the DatabaseObject
     * @see DatabaseObject.get - gets a value from the DatabaseObject
     * @see DatabaseObject.update - updates a value from the DatabaseObject
     *
     */
    contains(key) {
        if (typeof key == "string")
            return this.contains(key.split(/[.\[\]]/g));
        else {
            if (key.length > 1) {
                let e = this._data[key[0]];
                if (!(e instanceof DatabaseObject || e instanceof DatabaseArray))
                    return false;
                return e === null || e === void 0 ? void 0 : e.contains(key.slice(1, key.length));
            }
            else {
                return this._data[key[0]] != null;
            }
        }
    }
}
exports.DatabaseObject = DatabaseObject;
/**
 * Database class is basically a DatabaseObject with save and load functionality
 *
 * @see SyncDatabase
 * @see Database.saveData
 * @see Database.reloadData
 *
 * @see DatabaseObject
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
class Database extends DatabaseObject {
    /**
     * Constructor for Database class
     *
     * @author Nicolas Schmidt
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see Database - 👩‍👦 the parent class
     */
    constructor(adapter, data) {
        super(data);
        this.adapter = adapter;
        this.adapter = adapter;
    }
    /**
     * Save the data from the database
     *
     * @author Nicolas Schmidt
     * @returns ⌛ Promise (Ready when saved) >> the database itself, so you can chain operations
     */
    saveData() {
        return __awaiter(this, void 0, void 0, function* () { yield this.adapter.save(this.data); return this; });
    }
    /**
     * Reload the data from the database
     *
     * @author Nicolas Schmidt
     * @returns ⌛ Promise (Ready when loaded) >> the database itself, so you can chain operations
     */
    reloadData() {
        return __awaiter(this, void 0, void 0, function* () { this.data = yield this.adapter.load(); return this; });
    }
}
exports.Database = Database;
/**
 * SyncDatabase is a database that does not use promises.
 *
 * @see Database
 * @see SyncDatabase.saveData
 * @see SyncDatabase.reloadData
 *
 * @see DatabaseObject
 * @see DatabaseObject.data
 * @see DatabaseObject.constructor
 * @see DatabaseObject.set
 * @see DatabaseObject.get
 * @see DatabaseObject.update
 */
class SyncDatabase extends DatabaseObject {
    /**
     * Constructor for Database class
     *
     * @author Nicolas Schmidt
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see Database - 👩‍👦 the parent class
     */
    constructor(adapter, data) {
        super(data || adapter.load());
        this.adapter = adapter;
        this.adapter = adapter;
    }
    /**
     * Save the data from the database
     *
     * @author Nicolas Schmidt
     * @returns the database itself, so you can chain operations
     */
    saveData() { this.adapter.save(this.data); return this; }
    /**
     * Reload the data from the database
     *
     * @author Nicolas Schmidt
     * @returns the database itself, so you can chain operations
     */
    reloadData() { this.data = this.adapter.load(); return this; }
}
exports.SyncDatabase = SyncDatabase;
function createDatabase(adapter) {
    let result = adapter.load();
    if (result instanceof Promise) {
        return new Promise((rs) => {
            result.then((data) => {
                rs(new Database(adapter, data));
            });
        });
    }
    return new SyncDatabase(adapter, result);
}
exports.createDatabase = createDatabase;
function createObjectForKey(key) {
    if (typeof key == "string")
        return createObjectForKey(key.split(/[.\[\]]/g));
    if (typeof key[1] == 'number' || (typeof key[1] == 'string' && isNumberString(key[1])))
        return new DatabaseArray([]);
    return new DatabaseObject();
}
const numberCharacters = "0123456789";
function isNumberString(s) {
    for (let i = 0; i < s.length; i++)
        if (!numberCharacters.includes(s.charAt(i)))
            return false;
    return true;
}
;
//# sourceMappingURL=src/sourcemaps/database.js.map