import { DatabaseAdapter, JSObject, SyncDatabaseAdapter } from './database';
/**
 * JsonFileAdapter is an async adapter for working with Json Files
 *
 * @author Nicolas Schmidt
 */
export declare class JsonFileAdapter implements DatabaseAdapter {
    /**
     * The path of the JsonFileAdapter
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    readonly path: string;
    /**
     * The settings for the JsonFileAdapter: <br/>
     *   beautify - should the generated json be beautified (default: false)<br/>
     *   beautify_space - the space to indent the json with (default: 2)
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    readonly settings: JsonAdapterSettings;
    /**
     * The constructor for JsonFileAdapter
     *
     * @param path the path to a json file to save to and load from
     * @param settings The settings for the JsonFileAdapter:<br/>
     *   beautify - should the generated json be beautified (default: false)<br/>
     *   beautify_space - the space to indent the json with (default: 2)
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    constructor(
    /**
     * The path of the JsonFileAdapter
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    path: string, settings?: JsonAdapterSettingsInput);
    /**
     * Saves the given data to the file the adapter refers to
     *
     * @param data The data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    save(data: JSObject): Promise<void>;
    /**
     * Loads data from the file the adapter refers to
     *
     * @returns ⌛ Promise (Ready when saved) >> Object
     *
     * @author Nicolas Schmidt
     *
     * @see JsonFileAdapter - 👩‍👦 the parent class
     */
    load(): Promise<JSObject>;
}
/**
 * SyncJsonFileAdapter is an sync adapter for working with Json Files
 *
 * @author Nicolas Schmidt
 */
export declare class SyncJsonFileAdapter implements SyncDatabaseAdapter {
    /**
     * The path of the SyncJsonFileAdapter
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    readonly path: string;
    /**
     * The settings for the SyncJsonFileAdapter: <br/>
     *   beautify - should the generated json be beautified (default: false)<br/>
     *   beautify_space - the space to indent the json with (default: 2)
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    readonly settings: JsonAdapterSettings;
    /**
     * The constructor for SyncJsonFileAdapter
     *
     * @param path the path to a json file to save to and load from
     * @param settings The settings for the SyncJsonFileAdapter:<br/>
     *   beautify - should the generated json be beautified (default: false)<br/>
     *   beautify_space - the space to indent the json with (default: 2)
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    constructor(
    /**
     * The path of the SyncJsonFileAdapter
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    path: string, settings?: JsonAdapterSettingsInput);
    /**
     * Saves the given data to the file the adapter refers to
     *
     * @param data The data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    save(data: JSObject): void;
    /**
     * Loads data from the file the adapter refers to
     *
     * @returns ⌛ Promise (Ready when saved) >> Object
     *
     * @author Nicolas Schmidt
     *
     * @see SyncJsonFileAdapter - 👩‍👦 the parent class
     */
    load(): JSObject;
}
/**
 * The settings for JsonFileAdapter or SyncJsonFileAdapter:
 *   beautify - should the generated json be beautified (default: false)
 *   beautify_space - the space to indent the json with (default: 2)
 *
 * This is the input version of the interface. It get's applied all default values when passed to a JsonFileAdapter or SyncJsonFileAdapter
 *
 * @author Nicolas Schmidt
 *
 * @see JsonFileAdapter
 * @see SyncJsonFileAdapter
 */
export interface JsonAdapterSettingsInput {
    beautify?: false;
    beautify_space?: number;
}
/**
 * The settings for JsonFileAdapter or SyncJsonFileAdapter:
 *   beautify - should the generated json be beautified (default: false)
 *   beautify_space - the space to indent the json with (default: 2)
 *
 * @author Nicolas Schmidt
 *
 * @see JsonFileAdapter
 * @see SyncJsonFileAdapter
 */
export interface JsonAdapterSettings {
    beautify: false;
    beautify_space: number;
}
